@@ -12,9 +12,12 @@
  */
 
 /*
- * $Id: branch.c,v 1.3 1997/10/23 04:19:38 mff Exp mff $
+ * $Id: branch.c,v 2.0 2001/01/10 12:12:37 gul Exp $
  *
  * $Log: branch.c,v $
+ * Revision 2.0  2001/01/10 12:12:37  gul
+ * Binkd is under CVS again
+ *
  * Revision 1.3  1997/10/23  04:19:38  mff
  * Amiga port, argument copying (DUP() removed)
  *
@@ -37,7 +40,9 @@
 #endif
 
 #ifdef HAVE_THREADS
+#ifndef __IBMC__
 #include <dos.h>
+#endif
 #include <process.h>
 #endif
 
@@ -77,6 +82,7 @@ int branch (register void (*F) (void *), register void *arg, register size_t siz
     arg = 0;
 
 #if defined(HAVE_FORK) && !defined(AMIGA)
+again:
   if (!(rc = fork ()))
   {
     /* new process */
@@ -85,6 +91,7 @@ int branch (register void (*F) (void *), register void *arg, register size_t siz
   }
   else if (rc < 0)
   {
+    if (errno == EINTR) goto again;
     /* parent, error */
     Log (1, "fork: %s", strerror (errno));
   }
