@@ -2,11 +2,28 @@
  * You must be VERY CAREFUL with this module. Note, this
  * code is working in VERY diff. ways in forking vs. threading versions!!
  */
+/*
+ * $Id: bsy.c,v 2.0 2001/01/10 12:12:37 gul Exp $
+ *
+ * $Log: bsy.c,v $
+ * Revision 2.0  2001/01/10 12:12:37  gul
+ * Binkd is under CVS again
+ *
+ *
+ */
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
 #include <time.h>
+#ifndef UNIX
+#include <io.h>
+#include <fcntl.h>
+#endif
+#ifdef OS2
+#define INCL_DOS
+#include <os2.h>
+#endif
 
 #include "Config.h"
 #include "ftnaddr.h"
@@ -28,6 +45,9 @@ struct _BSY_ADDR
   BSY_ADDR *next;
   FTN_ADDR fa;
   bsy_t bt;
+#ifndef UNIX
+  int h;
+#endif
 };
 
 BSY_ADDR *bsy_list = 0;
@@ -81,6 +101,19 @@ int bsy_add (FTN_ADDR *fa0, bsy_t bt)
 
       new_bsy->bt = bt;
 
+#ifndef UNIX
+      new_bsy->h = open(buf, O_RDONLY);
+      if (new_bsy->h == -1)
+        Log (2, "Can't open %s: %s!", buf, strerror(errno));
+#if defined(OS2)
+      else
+        DosSetFHState(new_bsy->h, OPEN_FLAGS_NOINHERIT);
+#elif defined(EMX)
+      else
+        fcntl(new_bsy->h,  F_SETFD, FD_CLOEXEC);
+#endif
+#endif
+
       ok = 1;
     }
   }
@@ -124,6 +157,11 @@ void bsy_remove (FTN_ADDR *fa0, bsy_t bt)
     {
       if (!ftnaddress_cmp (&bsy->fa, fa0) && bsy->bt == bt)
       {
+#ifndef UNIX
+	if (bsy->h != -1)
+	  if (close(bsy->h))
+            Log (2, "Can't close %s (handle %d): %s!", buf, bsy->h, strerror(errno));
+#endif
 	delete (buf);
 	FA_ZERO (&bsy->fa);
 	break;
@@ -147,6 +185,11 @@ void bsy_remove_all ()
     if (*buf)
     {
       strnzcat (buf, bsy->bt == F_CSY ? ".csy" : ".bsy", sizeof (buf));
+#ifndef UNIX
+      if (bsy->h != -1)
+        if (close(bsy->h))
+          Log (2, "Can't close %s (handle %d): %s!", buf, bsy->h, strerror(errno));
+#endif
       delete (buf);
       FA_ZERO (&bsy->fa);
     }
@@ -162,6 +205,7 @@ void bsy_touch ()
 {
   static time_t last_touch = 0;
 
+  LockSem (&sem);
   if (time (0) - last_touch > BSY_TOUCH_DELAY)
   {
     BSY_ADDR *bsy;
@@ -181,4 +225,5 @@ void bsy_touch ()
     }
     last_touch = time (0);
   }
+  ReleaseSem (&sem);
 }
