@@ -1,3 +1,12 @@
+/*
+ * $Id: bsy.h,v 2.0 2001/01/10 12:12:37 gul Exp $
+ *
+ * $Log: bsy.h,v $
+ * Revision 2.0  2001/01/10 12:12:37  gul
+ * Binkd is under CVS again
+ *
+ *
+ */
 #ifndef _bsy_h
 #define _bsy_h
 
@@ -10,7 +19,7 @@ typedef unsigned char bsy_t;
 
 /*
  */
-void bsy_init();
+void bsy_init(void);
 
 /*
  * Test & add a busy-flag. 1 -- ok, 0 -- failed
@@ -29,12 +38,12 @@ void bsy_remove(FTN_ADDR *fa, bsy_t bt);
 /*
  * For exitlist...
  */
-void bsy_remove_all();
+void bsy_remove_all(void);
 
 /*
  * Touchs all our .bsy's if needed
  */
-void bsy_touch ();
+void bsy_touch (void);
 #define BSY_TOUCH_DELAY 60
 
 #endif
