@@ -12,9 +12,12 @@
  */
 
 /*
- *  $Id: exitproc.c,v 1.2 1997/10/23 04:13:35 mff Exp mff $
+ * $Id: exitproc.c,v 2.0 2001/01/10 12:12:37 gul Exp $
+ *
+ * $Log: exitproc.c,v $
+ * Revision 2.0  2001/01/10 12:12:37  gul
+ * Binkd is under CVS again
  *
- *  $Log: exitproc.c,v $
  * Revision 1.2  1997/10/23  04:13:35  mff
  * pidfiles are now killed only by servmgrs, misc
  *
@@ -39,8 +42,9 @@ void exitfunc (void)
 {
 #ifdef HAVE_FORK
   if (pidcmgr)
-  {
-    kill (pidcmgr, SIGTERM);
+  { int i;
+    i=pidcmgr, pidcmgr=0; /* prevent abort when cmgr exits */
+    kill (i, SIGTERM);
     sleep (1);
   }
 #endif
