@@ -12,9 +12,12 @@
  */
 
 /*
- * $Id: ftnaddr.c,v 1.4 1997/10/23 04:11:58 mff Exp mff $
+ * $Id: ftnaddr.c,v 2.0 2001/01/10 12:12:37 gul Exp $
  *
  * $Log: ftnaddr.c,v $
+ * Revision 2.0  2001/01/10 12:12:37  gul
+ * Binkd is under CVS again
+ *
  * Revision 1.4  1997/10/23  04:11:58  mff
  * +ftnaddress_to_domain()
  */
@@ -65,7 +68,8 @@ static int gettoken (TOKEN *token, char **s)
   {
     int i;
 
-    for (i = 0; i < MAX_DOMAIN && (**s == '.' || isalnum (**s)); ++i)
+    for (i = 0; i < MAX_DOMAIN &&
+         (**s == '.' || **s == '-' || **s == '_' || isalnum (**s)); ++i)
       token->s[i] = *((*s)++);
     token->s[i] = 0;
   }
@@ -235,13 +239,9 @@ void ftnaddress_to_filename (char *s, FTN_ADDR *fa)
       sprintf (pnt, ".pnt%s%08x", PATH_SEPARATOR, fa->p);
 
 #ifdef HAVE_SNPRINTF
-    snprintf
+    snprintf(s, MAXPATHLEN,
 #else
-    sprintf
-#endif
-      (s,
-#ifdef HAVE_SNPRINTF
-       MAXPATHLEN,
+    sprintf(s,
 #endif
        "%s%s%s%s%s%04x%04x%s",
        d->path, PATH_SEPARATOR, d->dir, ext, PATH_SEPARATOR,
