@@ -12,9 +12,15 @@
  */
 
 /*
- * $Id: ftnnode.h,v 1.6 1997/11/03 06:10:39 mff Exp mff $
+ * $Id: ftnnode.h,v 2.1 2001/02/15 11:03:18 gul Exp $
  *
  * $Log: ftnnode.h,v $
+ * Revision 2.1  2001/02/15 11:03:18  gul
+ * Added crypt traffic possibility
+ *
+ * Revision 2.0  2001/01/10 12:12:38  gul
+ * Binkd is under CVS again
+ *
  * Revision 1.6  1997/11/03  06:10:39  mff
  * +nodes_init()
  *
@@ -55,17 +61,24 @@ struct _FTN_NODE
   char *obox;
   char *ibox;
   int NR_flag;
+  int ND_flag;
+  int MD_flag;
+  int crypt_flag;
+  int restrictIP;
 
   time_t hold_until;
   int busy;			       /* 0=free, 'c'=.csy, other=.bsy */
   int mail_flvr;		       /* -1=no mail, other=it's flavour */
   int files_flvr;		       /* -1=no files, other=it's flavour */
+#ifdef HTTPS
+  unsigned long current_addr;
+#endif
 };
 
 /*
  * Call this before all others functions from this file.
  */
-void nodes_init ();
+void nodes_init (void);
 
 /*
  * Return up/downlink info by fidoaddress. 0 == node not found
@@ -83,16 +96,25 @@ int node_cmp (FTN_NODE *a, FTN_NODE *b);
  * 1 -- ok, 0 -- error;
  */
 int add_node (FTN_ADDR *fa, char *hosts, char *pwd, char obox_flvr,
-	       char *obox, char *ibox, int NR_flag);
+	      char *obox, char *ibox, int NR_flag, int ND_flag,
+	      int crypt_flag, int MD_flag, int restrictIP);
 
 #define NR_ON       1
 #define NR_OFF      0
 #define NR_USE_OLD -1		       /* Use old value */
 
+#define ND_ON       1
+#define ND_OFF      0
+#define ND_USE_OLD -1		       /* Use old value */
+
+#define CRYPT_ON	 1
+#define CRYPT_OFF	 0
+#define CRYPT_USE_OLD	-1		/* Use old value */
+
 /*
  * Iterates through nodes while func() == 0.
  */
-int foreach_node (int (*func) (FTN_NODE *, void *), void *);
+int foreach_node (int (*func) (FTN_NODE * a1, void * a2), void * a3);
 
 /*
  * Dump node list
