@@ -12,9 +12,12 @@
  */
 
 /*
- * $Id: ftnq.c,v 1.14 1998/05/05 04:23:52 mff Exp mff $
+ * $Id: ftnq.c,v 2.0 2001/01/10 12:12:38 gul Exp $
  *
  * $Log: ftnq.c,v $
+ * Revision 2.0  2001/01/10 12:12:38  gul
+ * Binkd is under CVS again
+ *
  * Revision 1.14  1998/05/05  04:23:52  mff
  * create_poll() now makes all the required intermediate
  * directories in the outbound.
@@ -164,7 +167,7 @@ FTNQ *q_scan (FTNQ *q)
 
 	  FA_ZERO (&fa);
 	  fa.z = ((de->d_name[len] == '.') ?
-		  strtol (de->d_name + len + 1, (char **) NULL, 16) :
+		  (int)strtol (de->d_name + len + 1, (char **) NULL, 16) :
 		  curr_domain->z[0]);
 	  if (de->d_name[len] == 0 || fa.z != curr_domain->z[0])
 	  {
@@ -224,14 +227,41 @@ static int cmp_filebox_files (FTNQ *a, FTNQ *b)
   int w_b = weight_by_name (b->path);
 
   if (w_a - w_b == 0)
-    return b->time - a->time;
+    return (int)(b->time - a->time);
   else
     return w_a - w_b;
 }
 
-FTNQ *q_scan_boxes (FTNQ *q, FTN_ADDR *fa, int n)
+#ifdef MAILBOX
+static char to32(int N)
+{
+    if ((N >= 0) && (N <=9)) return (char)('0'+N);
+    if ((N > 9) && (N < 32)) return (char)('A'+ N - 10);
+    return 'Z'; /* fake return */
+};
+
+static struct {
+    char * ext;
+    char   flv;
+} brakeExt[] =
+{
+#ifdef UNIX
+    {"immediate", 'i'},
+    {"crash",     'c'},
+    {"direct",    'd'},
+    {"normal",    'f'},
+    {"hold",      'h'},
+#endif
+    {"Immediate", 'i'},
+    {"Crash",     'c'},
+    {"Direct",    'd'},
+    {"Normal",    'f'},
+    {"Hold",      'h'}
+};
+#endif
+
+static FTNQ *q_scan_box (FTNQ *q, FTN_ADDR *fa, char *boxpath, char flvr, int deleteempty)
 {
-  FTN_NODE *node;
   FTNQ *files = NULL;
   int n_files = 0, i;
   DIR *dp;
@@ -239,36 +269,38 @@ FTNQ *q_scan_boxes (FTNQ *q, FTN_ADDR *fa, int n)
   struct dirent *de;
   struct stat sb;
 
-  for (i = 0; i < n; ++i)
+  strnzcpy (buf, boxpath, sizeof (buf));
+  strnzcat (buf, PATH_SEPARATOR, sizeof (buf));
+  s = buf + strlen (buf);
+  if ((dp = opendir (boxpath)) != NULL)
   {
-    if ((node = get_node_info (fa + i)) != NULL && node->obox != NULL)
+    while ((de = readdir (dp)) != 0)
     {
-      strnzcpy (buf, node->obox, sizeof (buf));
-      strnzcat (buf, PATH_SEPARATOR, sizeof (buf));
-      s = buf + strlen (buf);
-      if ((dp = opendir (node->obox)) != NULL)
+      strnzcat (buf, de->d_name, sizeof (buf));
+      if (stat (buf, &sb) == 0 && (sb.st_mode & S_IFDIR) == 0 &&
+	  de->d_name[0] != '.'
+#if defined(OS2) && !defined(__IBMC__)
+          && (de->d_attr & 0x02) == 0   /* not hidden */
+#endif
+	 )
       {
-	while ((de = readdir (dp)) != 0)
-	{
-	  strnzcat (buf, de->d_name, sizeof (buf));
-	  if (stat (buf, &sb) == 0 && (sb.st_mode & S_IFDIR) == 0 &&
-	      de->d_name[0] != '.')
-	  {
-	    /* q = q_add_file (q, buf, fa + i, node->obox_flvr, 'd', 0); */
-	    files = xrealloc (files, sizeof (FTNQ) * (n_files + 1));
-	    strcpy (files[n_files].path, buf);
-	    files[n_files].time = sb.st_mtime;
-	    memcpy (&files[n_files].fa, fa + i, sizeof (FTN_ADDR));
-	    files[n_files].flvr = node->obox_flvr;
-	    ++n_files;
-	  }
-	  *s = 0;
-	}
-	closedir (dp);
+	files = xrealloc (files, sizeof (FTNQ) * (n_files + 1));
+	strcpy (files[n_files].path, buf);
+	files[n_files].time = sb.st_mtime;
+	memcpy (&files[n_files].fa, fa, sizeof (FTN_ADDR));
+	files[n_files].flvr = flvr;
+	++n_files;
       }
+      *s = 0;
+    }
+    closedir (dp);
+    if (n_files == 0 && deleteempty) {
+      if (rmdir (boxpath) == 0)
+        Log (3, "Empty filebox %s deleted", boxpath);
+      else
+        Log (1, "Cannot delete empty filebox %s: %s", boxpath, strerror (errno));
     }
   }
-
   if (files)
   {
     qsort (files, n_files, sizeof (FTNQ),
@@ -283,60 +315,106 @@ FTNQ *q_scan_boxes (FTNQ *q, FTN_ADDR *fa, int n)
 /*
  * Scans fileboxes for n akas stored in fa
  */
-/*
 FTNQ *q_scan_boxes (FTNQ *q, FTN_ADDR *fa, int n)
 {
+  FTN_NODE *node;
   int i;
+#ifdef MAILBOX
+  char buf[MAXPATHLEN + 1];
+  char *s;
+  int j;
+#endif
 
   for (i = 0; i < n; ++i)
   {
-    FTN_NODE *node = get_node_info (fa + i);
-
-    if (node && node->obox)
+#ifndef MAILBOX
+    if ((node = get_node_info (fa + i)) != NULL && node->obox != NULL)
     {
-      DIR *dp;
-      char buf[MAXPATHLEN + 1], *s;
+      q = q_scan_box (q, fa+i, node->obox, node->obox_flvr, 0);
+    }
+#else
+    if ((node = get_node_info (fa + i)) != NULL && ((node->obox != NULL) || tfilebox[0] || bfilebox[0]))
+    {
+      if (node->obox)
+        q = q_scan_box (q, fa+i, node->obox, node->obox_flvr, 0);
+      if (bfilebox[0]) {
+        strnzcpy (buf, bfilebox, sizeof (buf));
+        strnzcat (buf, PATH_SEPARATOR, sizeof (buf));
+#ifdef HAVE_SNPRINTF
+        snprintf(buf + strlen(buf), sizeof(buf) - strlen(buf),
+#else
+        sprintf(buf + strlen(buf),
+#endif
+                "%s.%u.%u.%u.%u.",
+                node->fa.domain,
+                node->fa.z,
+                node->fa.net,
+                node->fa.node,
+                node->fa.p);
+        s = buf + strlen(buf);
+        for (j = 0; j < sizeof(brakeExt)/sizeof(brakeExt[0]); j++) {
+          strnzcat (buf, brakeExt[j].ext, sizeof (buf));
+          q = q_scan_box (q, fa+i, buf, brakeExt[j].flv, deleteablebox);
+          *s = '\0';
+        }
+      }
+        
+      if (tfilebox[0]) {
+        strnzcpy ( buf, tfilebox, sizeof (buf));
+        strnzcat ( buf, PATH_SEPARATOR, sizeof (buf));
+#ifdef HAVE_SNPRINTF
+        snprintf(buf + strlen(buf), sizeof(buf) - strlen(buf),
+#else
+        sprintf(buf + strlen(buf),
+#endif
+                "%u.%u.%u.%u",
+                node->fa.z,
+                node->fa.net,
+                node->fa.node,
+                node->fa.p);
+        q = q_scan_box (q, fa+i, buf, 'f', deleteablebox);
+        strnzcat ( buf, ".h", sizeof (buf));
+        q = q_scan_box (q, fa+i, buf, 'h', deleteablebox);
 
-      strnzcpy (buf, node->obox, sizeof (buf));
-      strnzcat (buf, PATH_SEPARATOR, sizeof (buf));
-      s = buf + strlen (buf);
-      if ((dp = opendir (node->obox)) != 0)
-      {
-	struct dirent *de;
-	struct stat sb;
-
-	while ((de = readdir (dp)) != 0)
-	{
-	  strnzcat (buf, de->d_name, sizeof (buf));
-	  if (stat (buf, &sb) == 0 && (sb.st_mode & S_IFDIR) == 0 &&
-	      de->d_name[0] != '.')
-	  {
-	    q = q_add_file (q, buf, fa + i, node->obox_flvr, 'd', 0);
-	  }
-	  *s = 0;
-	}
-	closedir (dp);
+        strnzcpy ( buf, tfilebox, sizeof (buf));
+        strnzcat ( buf, PATH_SEPARATOR, sizeof (buf));
+#ifdef HAVE_SNPRINTF
+        snprintf(buf + strlen(buf), sizeof(buf) - strlen(buf),
+#else
+        sprintf(buf + strlen(buf),
+#endif
+                "%c%c%c%c%c%c%c%c.%c%c",
+                to32(node->fa.z/32),     to32(node->fa.z%32),
+                to32(node->fa.net/1024), to32((node->fa.net/32)%32), to32(node->fa.net%32),
+                to32(node->fa.node/1024),to32((node->fa.node/32)%32),to32(node->fa.node%32),
+                to32(node->fa.p/32),     to32(node->fa.p%32));
+        q = q_scan_box (q, fa+i, buf, 'f', deleteablebox);
+        strnzcat (buf, "h", sizeof (buf));
+        q = q_scan_box (q, fa+i, buf, 'h', deleteablebox);
       }
     }
+#endif
   }
+
   return q;
 }
-*/
 
 
 void process_hld (FTN_ADDR *fa, char *path)
 {
   FTN_NODE *node = get_node_info (fa);
+  long hold_until_tmp;
 
   if (node)
   {
     FILE *f;
 
     if ((f = fopen (path, "r")) == NULL ||
-	fscanf (f, "%ld", &node->hold_until) != 1)
+	fscanf (f, "%ld", &hold_until_tmp) != 1)
     {
       node->hold_until = 0;
     }
+    node->hold_until = (time_t)hold_until_tmp;
     if (f)
       fclose (f);
 
@@ -456,6 +534,11 @@ static FTNQ *q_add_dir (FTNQ *q, char *dir, FTN_ADDR *fa1)
 		/* Adding *.?lo */
 		q = q_add_file (q, buf, &fa2, s[9], 'd', 'l');
 	      }
+	      else if (!STRICMP (s + 9, "stc"))
+	      {
+		/* Adding *.stc */
+		q = q_add_file (q, buf, &fa2, 'h', 0, 's');
+	      }
 	    }
 	  }
 	}
@@ -473,11 +556,44 @@ static FTNQ *q_add_dir (FTNQ *q, char *dir, FTN_ADDR *fa1)
  */
 FTNQ *q_add_file (FTNQ *q, char *filename, FTN_ADDR *fa1, char flvr, char action, char type)
 {
+  const int argc=3;
+  char *argv[3];
+  char str[MAXPATHLEN+1];
+
   if (q != SCAN_LISTED)
   {
-    FTNQ *new_file = xalloc (sizeof (FTNQ));
+    FTNQ *new_file;
 
+    if (type == 's')
+    { char *p;
+      FILE *f;
+
+      f = fopen(filename, "r");
+      if (f == NULL)
+      { Log(1, "Can't open %s: %s", filename, strerror(errno));
+        return q;
+      }
+      if (!fgets(str, sizeof(str), f))
+      { Log(1, "Incorrect status (can't fgets), ignored");
+        fclose(f);
+        return q;
+      }
+      fclose(f);
+      if (*str && isspace(*str))
+      { Log(1, "Incorrect status (space first), ignored");
+        return q;
+      }
+      for (p=str+strlen(str)-1; isspace(*p); *p--='\0');
+      Log(2, "Status is '%s'", str);
+      if (!parse_args (argc, argv, str, "Status"))
+      { Log(1, "Incorrect status, ignored");
+        return q;
+      }
+    }
+
+    new_file = xalloc (sizeof (FTNQ));
     FQ_ZERO (new_file);
+
     new_file->next = q;
     if (q)
       q->prev = new_file;
@@ -486,11 +602,18 @@ FTNQ *q_add_file (FTNQ *q, char *filename, FTN_ADDR *fa1, char flvr, char action
     if (fa1)
       memcpy (&q->fa, fa1, sizeof (FTN_ADDR));
 
-    strnzcpy (q->path, filename, MAXPATHLEN);
     q->flvr = flvr;
     q->action = action;
     q->type = type;
     q->sent = 0;
+
+    if (type == 's')
+    { q->size = atol(argv[1]);
+      q->time = atol(argv[2]);
+      strnzcpy (q->path, argv[0], MAXPATHLEN);
+    }
+    else
+      strnzcpy (q->path, filename, MAXPATHLEN);
   }
   else
   {
@@ -520,14 +643,20 @@ typedef struct
 static int qn_list (FTN_NODE *fn, void *arg)
 {
   char tmp[60], buf[FTN_ADDR_SZ + 1];
-  struct tm *tm;
   qn_list_arg *a = (qn_list_arg *) arg;
 
   if (fn->mail_flvr || fn->files_flvr || fn->busy)
   {
     if (fn->hold_until > 0)
     {
+      struct tm *tm;
+#ifdef __WATCOMC__
+      struct tm stm;
+      tm = &stm;
+      _localtime (&fn->hold_until, tm);
+#else
       tm = localtime (&fn->hold_until);
+#endif
       strftime (tmp, sizeof (tmp), " (hold until %Y/%m/%d %H:%M:%S)", tm);
     }
     else
@@ -583,6 +712,16 @@ FTNQ *select_next_file (FTNQ *q, FTN_ADDR *fa, int nAka)
 
   for (k = 0; k < nAka; ++k)
   {
+    for (curr = q; curr; curr = curr->next)	/* Status first */
+    {
+      if (!curr->sent &&
+	  (FA_ISNULL (&curr->fa) || !ftnaddress_cmp (&curr->fa, fa + k)) &&
+	  curr->type == 's')
+      {
+	curr->sent = 1;
+	return curr;
+      }
+    }
     for (j = 0; prio[j]; ++j)
     {
       for (curr = q; curr; curr = curr->next)	/* Netmail first */
@@ -614,6 +753,43 @@ FTNQ *select_next_file (FTNQ *q, FTN_ADDR *fa, int nAka)
   return 0;
 }
 
+/*
+ * get size of files in the queue
+ */
+void q_get_sizes (FTNQ *q, unsigned long *netsize, unsigned long *filessize)
+{
+  FTNQ *curr;
+  struct stat st;
+  char *p;
+
+  *netsize = *filessize = 0;
+  for (curr = q; curr; curr = curr->next)
+  {
+    if (curr->type == 'l')
+    { FILE *f;
+      char str[MAXPATHLEN+2];
+
+      if ((f = fopen(curr->path, "r")) != NULL)
+      {
+        while (fgets (str, sizeof(str), f))
+        {
+          if (*str == '~' || *str == '$') continue;
+          if ((p = strchr(str, '\n')) != NULL) *p='\0';
+          p=str;
+          if (*str == '#' || *str == '^') p++;
+          if (stat(p, &st) == 0)
+            *filessize += st.st_size;
+        }
+        fclose(f);
+      }
+    }
+    else if (curr->type == 's')
+      *filessize += curr->size;
+    else if (stat(curr->path, &st) == 0)
+      *(curr->type == 'm' ? netsize : filessize) += st.st_size;
+  }
+}
+
 /*
  * q_not_empty () == 0: the queue is empty.
  */
@@ -678,14 +854,15 @@ int create_poll (FTN_ADDR *fa, int flvr)
 
   strcpy (ext, ".flo");
   if (flvr && strchr (flo_flvrs, flvr))
-    ext[1] = tolower (flvr);
+    ext[1] = (char) tolower (flvr);
   ftnaddress_to_filename (buf, fa);
   if (*buf)
   {
     mkpath (buf);
     strnzcat (buf, ext, sizeof (buf));
     if ((rc = create_empty_sem_file (buf)) == 0)
-      Log (1, "cannot create %s: %s", buf, strerror (errno));
+      if (errno != EEXIST)
+        Log (1, "cannot create %s: %s", buf, strerror (errno));
   }
   else
     Log (1, "%s: unknown domain", fa->domain);
@@ -702,7 +879,13 @@ void hold_node (FTN_ADDR *fa, time_t hold_until)
   char time[80];
   struct tm *tm;
 
+#ifdef __WATCOMC__
+  struct tm stm;
+  tm = &stm;
+  _localtime (&hold_until, tm);
+#else
   tm = localtime (&hold_until);
+#endif
   strftime (time, sizeof (time), "%Y/%m/%d %H:%M:%S", tm);
   ftnaddress_to_str (addr, fa);
   ftnaddress_to_filename (buf, fa);
