@@ -11,9 +11,12 @@
  *  (at your option) any later version. See COPYING.
  */
 
-/* $Id: ftnq.h,v 1.8 1998/05/05 23:59:33 mff Exp mff $
+/* $Id: ftnq.h,v 2.0 2001/01/10 12:12:38 gul Exp $
  *
  * $Log: ftnq.h,v $
+ * Revision 2.0  2001/01/10 12:12:38  gul
+ * Binkd is under CVS again
+ *
  * Revision 1.8  1998/05/05  23:59:33  mff
  * Added comment about FTNQ.time.
  *
@@ -32,7 +35,7 @@
 #ifndef _ftnq_h
 #define _ftnq_h
 
-#if !defined(NT) && !defined(W95)
+#if !defined(WIN32) && !defined(DOS)
 #include <sys/time.h>
 #endif
 
@@ -95,13 +98,13 @@ FTNQ *q_scan_boxes (FTNQ *q, FTN_ADDR *fa, int n);
 /*
  * 0 = the queue is empty.
  */
-FTN_NODE *q_not_empty ();
+FTN_NODE *q_not_empty (void);
 
 /*
  * Selects a node to make the next call. (It's alost like
  * q_not_empty(), but it will never select a node twice)
  */
-FTN_NODE *q_next_node ();
+FTN_NODE *q_next_node (void);
 
 /*
  * Selects from q the next file for fa (Returns a pointer to a q element)
@@ -123,6 +126,11 @@ int create_poll (FTN_ADDR *fa, int flvr);
  */
 void hold_node (FTN_ADDR *fa, time_t hold_until);
 
+/*
+ * get size of files in the queue
+ */
+void q_get_sizes (FTNQ *q, unsigned long *netsize, unsigned long *filessize);
+
 #define FQ_ZERO(x) (memset(x, 0, sizeof(*(x))), FA_ZERO(&((x)->fa)))
 #define FQ_ISNULL(x) (FA_ISNULL(&((x)->fa)))
 
@@ -137,7 +145,7 @@ extern const char prio[];
 /*
  * Compare flavours. `I' is the best, `\0' is the worst.
  */
-#define MAXFLVR(a,b) ((strchr(prio, (a)) < strchr(prio, (b))) ? (a) : (b))
+#define MAXFLVR(a,b) ((strchr(prio,(a)) < strchr(prio,(b))) ? (a) : (b))
 
 void bad_try (FTN_ADDR *fa, const char *error);
 void good_try (FTN_ADDR *fa, char *comment);
