@@ -12,9 +12,15 @@
  */
 
 /*
- * $Id: inbound.c,v 1.10 1997/10/23 04:03:07 mff Exp mff $
+ * $Id: inbound.c,v 2.1 2001/04/23 07:58:57 gul Exp $
  *
  * $Log: inbound.c,v $
+ * Revision 2.1  2001/04/23 07:58:57  gul
+ * getfree() on large drives fixed
+ *
+ * Revision 2.0  2001/01/10 12:12:38  gul
+ * Binkd is under CVS again
+ *
  * Revision 1.10  1997/10/23  04:03:07  mff
  * minor fixes
  *
@@ -66,7 +72,7 @@ static void remove_hr (char *path)
   }
 }
 
-static int creat_tmp_name (char *s, char *file, size_t size,
+static int creat_tmp_name (char *s, char *file, off_t size,
 			    time_t time, FTN_ADDR *from, char *inbound)
 {
   FILE *f;
@@ -99,44 +105,6 @@ static int creat_tmp_name (char *s, char *file, size_t size,
   return 1;
 }
 
-/* returns 1 if the file is complete */
-static int completion_test (char *name)
-{
-  FILE *f;
-  char buf[MAXPATHLEN + 80];
-  int retval = 0;
-
-  if ((f = fopen (name, "r")) != 0)
-  {
-    if (fgets (buf, sizeof (buf), f))
-    {
-      char *w;
-
-      if ((w = getwordx (buf, 2, GWX_NOESC)) != 0)
-      {
-	struct stat sb;
-	int rc;
-	size_t size = (size_t) atol (w);
-
-	free (w);
-	strcpy (strrchr (name, '.'), ".dt");
-	rc = stat (name, &sb);
-	strcpy (strrchr (name, '.'), ".hr");
-
-	if (rc == 0 && (size_t) sb.st_size == size)
-	  retval = 1;
-      }
-    }
-    fclose (f);
-  }
-  if (retval)
-  {
-    Log (4, "the file is complete, but not renamed to it's realname,");
-    Log (4, "so I'll not remove it!");
-  }
-  return retval;
-}
-
 static int to_be_deleted (char *tmp_name, char *netname)
 {
   struct stat sb;
@@ -146,8 +114,7 @@ static int to_be_deleted (char *tmp_name, char *netname)
       time (0) - sb.st_mtime > kill_old_partial_files)
   {
     Log (4, "found old .dt/.hr files for %s", netname);
-    strcpy (strrchr (tmp_name, '.'), ".hr");
-    return !completion_test (tmp_name);
+    return 1;
   }
   strcpy (strrchr (tmp_name, '.'), ".hr");
   return 0;
@@ -157,8 +124,8 @@ static int to_be_deleted (char *tmp_name, char *netname)
  * Searches for the ``file'' in the inbound and returns it's tmp name in s.
  * S must have MAXPATHLEN chars. Returns 0 on error, 1=found, 2=created.
  */
-int find_tmp_name (char *s, char *file, size_t size,
-		    time_t time, FTN_ADDR *from, char *inbound)
+int find_tmp_name (char *s, char *file, off_t size,
+		    time_t time, FTN_ADDR *from, int nfa, char *inbound)
 {
   char buf[MAXPATHLEN + 80];
   DIR *dp;
@@ -167,6 +134,9 @@ int find_tmp_name (char *s, char *file, size_t size,
   int i, found = 0;
   char *t;
 
+  if (temp_inbound[0])
+    inbound = temp_inbound;
+
   if ((dp = opendir (inbound)) == 0)
   {
     Log (1, "cannot opendir %s: %s", inbound, strerror (errno));
@@ -200,15 +170,18 @@ int find_tmp_name (char *s, char *file, size_t size,
       for (i = 0; i < 4; ++i)
 	w[i] = getwordx (buf, i + 1, GWX_NOESC);
 
-      if (!strcmp (w[0], file))
+      if (!strcmp (w[0], file) && parse_ftnaddress (w[3], &fa))
       {
-	if (size == (size_t) atol (w[1]) &&
+	for (i = 0; i < nfa; i++)
+	  if (!ftnaddress_cmp (&fa, from + i))
+	    break;
+	if (size == (off_t) atol (w[1]) &&
 	    (time & ~1) == (atol (w[2]) & ~1) &&
-	    parse_ftnaddress (w[3], &fa) && !ftnaddress_cmp (&fa, from))
+	    i < nfa)
 	{
 	  found = 1;
 	}
-	else if (kill_dup_partial_files)
+	else if (kill_dup_partial_files && i < nfa)
 	{
 	  Log (4, "dup partial file (%s):", w[0]);
 	  remove_hr (s);
@@ -245,20 +218,26 @@ int find_tmp_name (char *s, char *file, size_t size,
   return found;
 }
 
-FILE *inb_fopen (char *netname, size_t size,
-		  time_t time, FTN_ADDR *from, char *inbound)
+FILE *inb_fopen (char *netname, off_t size,
+		  time_t time, FTN_ADDR *from, int nfa, char *inbound)
 {
   char buf[MAXPATHLEN + 1];
   struct stat sb;
   FILE *f;
 
-  if (!find_tmp_name (buf, netname, size, time, from, inbound))
+  if (!find_tmp_name (buf, netname, size, time, from, nfa, inbound))
     return 0;
 
   if ((f = fopen (buf, "ab")) == 0)
     Log (1, "%s: %s", buf, strerror (errno));
   fseek (f, 0, SEEK_END);	       /* Work-around MSVC bug */
 
+#if defined(OS2)
+  DosSetFHState(fileno(f), OPEN_FLAGS_NOINHERIT);
+#elif defined(EMX)
+  fcntl(fileno(f), F_SETFD, FD_CLOEXEC);
+#endif
+
   /* Checking for free space */
   if (fstat (fileno (f), &sb) == 0)
   {
@@ -267,12 +246,12 @@ FILE *inb_fopen (char *netname, size_t size,
 		    minfree : minfree_nonsecure);
 
     if (req_free >= 0 &&
-	getfree (inbound) < size - sb.st_size + req_free * 1024)
+	getfree (inbound) < (unsigned long)((size - sb.st_size + 1023) / 1024 + req_free))
     {
-      Log (1, "no enough free space in %s (%li, req-d %li)",
+      Log (1, "no enough free space in %s (%liK, req-d %liK)",
 	   inbound,
 	   (long) getfree (inbound),
-	   (long) size - sb.st_size + req_free * 1024l);
+	   (long) (size - sb.st_size + 1023) / 1024 + req_free);
       fclose (f);
       return 0;
     }
@@ -283,12 +262,12 @@ FILE *inb_fopen (char *netname, size_t size,
   return f;
 }
 
-int inb_reject (char *netname, size_t size,
-		 time_t time, FTN_ADDR *from, char *inbound)
+int inb_reject (char *netname, off_t size,
+		 time_t time, FTN_ADDR *from, int nfa, char *inbound)
 {
   char tmp_name[MAXPATHLEN + 1];
 
-  if (find_tmp_name (tmp_name, netname, size, time, from, inbound) != 1)
+  if (find_tmp_name (tmp_name, netname, size, time, from, nfa, inbound) != 1)
   {
     Log (1, "missing tmp file for %s!", netname);
     return 0;
@@ -307,15 +286,15 @@ int inb_reject (char *netname, size_t size,
  * File is complete, rename it to it's realname. 1=ok, 0=failed.
  * Sets realname[MAXPATHLEN]
  */
-int inb_done (char *netname, size_t size,
-	       time_t time, FTN_ADDR *from, char *inbound, char *real_name)
+int inb_done (char *netname, off_t size, time_t time,
+	      FTN_ADDR *from, int nfa, char *inbound, char *real_name)
 {
   char tmp_name[MAXPATHLEN + 1];
   char *s, *u;
 
   *real_name = 0;
 
-  if (find_tmp_name (tmp_name, netname, size, time, from, inbound) != 1)
+  if (find_tmp_name (tmp_name, netname, size, time, from, nfa, inbound) != 1)
   {
     Log (1, "missing tmp file for %s!", netname);
     return 0;
@@ -334,6 +313,9 @@ int inb_done (char *netname, size_t size,
   if (ispkt (netname) || istic (netname) || isarcmail (netname))
     s -= 4;
 
+  if (touch (tmp_name, time) != 0)
+    Log (1, "touch %s: %s", tmp_name, strerror (errno));
+
   while (1)
   {
     if (!RENAME (tmp_name, real_name))
@@ -342,13 +324,22 @@ int inb_done (char *netname, size_t size,
       break;
     }
     else
-      Log (5, "error renaming `%s' to `%s': %s",
+    {
+      if (errno != EEXIST && errno != EACCES && errno != EAGAIN)
+      {
+        Log (1, "cannot rename %s to it's realname: %s! (data stored in %s)",
+             netname, strerror (errno), tmp_name);
+        *real_name = 0;
+        return 0;
+      }
+      Log (2, "error renaming `%s' to `%s': %s",
 	   netname, real_name, strerror (errno));
+    }
 
     if (isalpha (*s) && toupper (*s) != 'Z')
-      ++* s;
+      ++*s;
     else if (isdigit (*s) && toupper (*s) != '9')
-      ++* s;
+      ++*s;
     else if (*s == '9')
       *s = 'a';
     else if (*--s == '.' || *s == '\\' || *s == '/')
@@ -360,9 +351,6 @@ int inb_done (char *netname, size_t size,
     }
   }
 
-  if (touch (real_name, time) == -1)
-    Log (1, "touch %s: %s", real_name, strerror (errno));
-
   /* Replacing .dt with .hr and removing temp. file */
   strcpy (strrchr (tmp_name, '.'), ".hr");
   delete (tmp_name);
@@ -373,7 +361,7 @@ int inb_done (char *netname, size_t size,
  * Checks if the file already exists in our inbound. !0=ok, 0=failed.
  * Sets realname[MAXPATHLEN]
  */
-int inb_test (char *filename, size_t size, time_t t,
+int inb_test (char *filename, off_t size, time_t t,
 	       char *inbound, char fp[])
 {
   char *s, *u;
@@ -386,6 +374,6 @@ int inb_test (char *filename, size_t size, time_t t,
   free (u);
   strwipe (s);
 
-  return stat (fp, &sb) == 0 && (size_t) sb.st_size == size &&
+  return stat (fp, &sb) == 0 &&  sb.st_size == size &&
     (sb.st_mtime & ~1) == (t & ~1);
 }
