@@ -12,9 +12,12 @@
  */
 
 /*
- * $Id: iptools.c,v 1.3 1998/06/19 05:19:33 mff Exp mff $
+ * $Id: iptools.c,v 2.0 2001/01/10 12:12:38 gul Exp $
  *
  * $Log: iptools.c,v $
+ * Revision 2.0  2001/01/10 12:12:38  gul
+ * Binkd is under CVS again
+ *
  * Revision 1.3  1998/06/19  05:19:33  mff
  * changes in get_hostname()
  *
@@ -31,30 +34,59 @@
 #include <fcntl.h>
 #include <ctype.h>
 
-#if !defined(NT) && !defined(W95)
+#if !defined(WIN32) && !defined(DOS)
 #include <sys/ioctl.h>
 #endif
 
-#include "iphdr.h"
-#include "iptools.h"
 #include "tools.h"
 #include "readcfg.h"
+#include "iphdr.h"
+#include "iptools.h"
 
 /*
  * Finds ASCIIZ address
  */
-const char *get_hostname (struct sockaddr_in * addr)
+const char *get_hostname (struct sockaddr_in *addr, char *host, int len)
 {
-  static char host[MAXHOSTNAMELEN + 1];
-  struct hostent *hp = ((backresolv == 0) ? 0 :
-			gethostbyaddr ((char *) &addr->sin_addr,
-				       sizeof addr->sin_addr,
-				       AF_INET));
+  struct hostent *hp;
+  struct sockaddr_in s;
+
+  memcpy(&s, addr, sizeof(s));
+  hp = ((backresolv == 0) ? 0 :
+	gethostbyaddr ((char *) &s.sin_addr,
+		       sizeof s.sin_addr,
+		       AF_INET));
+
+  strnzcpy (host, hp ? hp->h_name : inet_ntoa (s.sin_addr), len);
 
-  strnzcpy (host, hp ? hp->h_name : inet_ntoa (addr->sin_addr), sizeof (host));
   return host;
 }
 
+#ifdef HAVE_THREADS
+void copy_hostent(struct hostent *dest, struct hostent *src)
+{
+  int naddr;
+  char **cp;
+
+  memcpy(dest, src, sizeof(struct hostent));
+  for (cp = src->h_addr_list, naddr = 0; cp && *cp; naddr++, cp++);
+  dest->h_addr_list = malloc((naddr+1)*sizeof(dest->h_addr_list[0]));
+  if (dest->h_addr_list)
+  {
+    dest->h_addr_list[0] = malloc(naddr*src->h_length);
+    if (dest->h_addr_list[0])
+    {
+      for (cp = src->h_addr_list, naddr=0; cp && *cp; cp++, naddr++)
+      {
+        dest->h_addr_list[naddr] = dest->h_addr_list[0]+naddr*src->h_length;
+        memcpy(dest->h_addr_list[naddr], *cp, src->h_length);
+      }
+      dest->h_addr_list[naddr] = NULL;
+    }
+  }
+}
+#endif
+
 /*
  * Sets non-blocking mode for a given socket
  */
@@ -69,7 +101,7 @@ void setsockopts (SOCKET s)
   if (ioctl (s, FIONBIO, (char *) &arg, sizeof arg) < 0)
     Log (1, "ioctl (FIONBIO): %s", TCPERR ());
 
-#elif defined(NT) || defined(W95)
+#elif defined(WIN32)
   u_long arg;
 
   arg = 1;
