@@ -12,9 +12,12 @@
  */
 
 /*
- * $Id: iptools.h,v 1.2 1997/10/23 04:00:57 mff Exp mff $
+ * $Id: iptools.h,v 2.0 2001/01/10 12:12:38 gul Exp $
  *
  * $Log: iptools.h,v $
+ * Revision 2.0  2001/01/10 12:12:38  gul
+ * Binkd is under CVS again
+ *
  * Revision 1.2  1997/10/23  04:00:57  mff
  * +find_port()
  *
@@ -26,7 +29,9 @@
 /*
  * Finds ASCIIZ address
  */
-const char *get_hostname (struct sockaddr_in * addr);
+const char *get_hostname (struct sockaddr_in * addr, char *host, int len);
+
+void copy_hostent(struct hostent *dest, struct hostent *src);
 
 /*
  * Sets non-blocking mode for a given socket
