@@ -12,9 +12,12 @@
  */
 
 /*
- * $Id: prothlp.c,v 1.3 1997/11/05 01:11:29 mff Exp mff $
+ * $Id: prothlp.c,v 2.0 2001/01/10 12:12:38 gul Exp $
  *
  * $Log: prothlp.c,v $
+ * Revision 2.0  2001/01/10 12:12:38  gul
+ * Binkd is under CVS again
+ *
  * Revision 1.3  1997/11/05  01:11:29  mff
  * bt+ arcshield
  *
@@ -39,16 +42,16 @@
 #include "srif.h"
 #include "assert.h"
 
-int tfile_cmp (TFILE *a, char *netname, size_t size, time_t time)
+int tfile_cmp (TFILE *a, char *netname, unsigned long size, time_t time)
 {
   int rc = strcmp (a->netname, netname);
 
   if (rc == 0)
   {
     if (a->size != size)
-      return a->size - size;
+      return (int)(a->size - size);
     if (a->time != time)
-      return a->time - time;
+      return (int)(a->time - time);
     return 0;
   }
   else
@@ -61,7 +64,7 @@ void add_to_killlist (KILLLIST **killlist, int *n_killlist, char *name, int cond
   *killlist = xrealloc (*killlist, sizeof (KILLLIST) * (*n_killlist + 1));
   memset (*killlist + *n_killlist, 0, sizeof (KILLLIST));
   strnzcpy ((*killlist)[*n_killlist].name, name, MAXPATHLEN);
-  (*killlist)[*n_killlist].cond = cond;
+  (*killlist)[*n_killlist].cond = (char)cond;
   ++*n_killlist;
 }
 
@@ -117,7 +120,7 @@ void free_rcvdlist (RCVDLIST **rcvdlist, int *n_rcvdlist)
 
 void netname (char *s, TFILE *q)
 {
-  static char *weekext[] = {"Su", "Mo", "Tu", "We", "Th", "Fr", "Sa"};
+  static char *weekext[] = {"su", "mo", "tu", "we", "th", "fr", "sa"};
 
   if (q->type == 'm')
   {
