@@ -12,9 +12,15 @@
  */
 
 /*
- * $Id: readcfg.c,v 1.14 1997/10/23 03:45:34 mff Exp mff $
+ * $Id: readcfg.c,v 2.1 2001/02/15 11:03:18 gul Exp $
  *
  * $Log: readcfg.c,v $
+ * Revision 2.1  2001/02/15 11:03:18  gul
+ * Added crypt traffic possibility
+ *
+ * Revision 2.0  2001/01/10 12:12:39  gul
+ * Binkd is under CVS again
+ *
  * Revision 1.14  1997/10/23  03:45:34  mff
  * +fdinhist, +fdouthist, +root_domain, many fixes to hide pNod into
  * ftnnode.c
@@ -79,6 +85,7 @@ static int line;
 
 char siport[MAXSERVNAME + 1] = "";
 char soport[MAXSERVNAME + 1] = "";
+int havedefnode=0;
 int iport = 0;
 int oport = 0;
 int call_delay = 60;
@@ -102,16 +109,31 @@ char location[MAXLOCATIONNAME + 1] = "";
 char nodeinfo[MAXNODEINFO + 1] = "";
 char inbound[MAXPATHLEN + 1] = ".";
 char inbound_nonsecure[MAXPATHLEN + 1] = "";
+char temp_inbound[MAXPATHLEN + 1] = "";
+#ifdef MAILBOX
+/* FileBoxes dir */
+char tfilebox[MAXPATHLEN + 1] = "";
+/* BrakeBoxes dir */
+char bfilebox[MAXPATHLEN + 1] = "";
+int  deleteablebox = 0;
+#endif
 char logpath[MAXPATHLEN + 1] = "";
 char binlogpath[MAXPATHLEN + 1] = "";
 char fdinhist[MAXPATHLEN + 1] = "";
 char fdouthist[MAXPATHLEN + 1] = "";
 char pid_file[MAXPATHLEN + 1] = "";
+#ifdef HTTPS
+char proxy[MAXHOSTNAMELEN + 40] = "";
+char socks[MAXHOSTNAMELEN + 40] = "";
+#endif
 int loglevel = 4;
 int conlog = 0;
 int send_if_pwd = 0;
 int tzoff = 0;
 char root_domain[MAXHOSTNAMELEN + 1] = "fidonet.net.";
+int prescan = 0;
+int connect_timeout = 0;
+struct conflist_type *config_list = NULL;
 
 #if defined (HAVE_VSYSLOG) && defined (HAVE_FACILITYNAMES)
 
@@ -134,10 +156,11 @@ struct _KEYWORD
   const char *key;
   void (*callback) (KEYWORD *key, char *s);
   void *var;
-  int option1;
-  int option2;
+  long option1;
+  long option2;
 };
 
+static void passwords (KEYWORD *, char *);
 static void include (KEYWORD *, char *);
 static void read_aka_list (KEYWORD *, char *);
 static void read_domain_info (KEYWORD *, char *);
@@ -147,6 +170,8 @@ static void read_string (KEYWORD *, char *);
 static void read_bool (KEYWORD *, char *);
 static void read_flag_exec_info (KEYWORD *, char *);
 static void read_rfrule (KEYWORD *, char *);
+static void read_skipmask (KEYWORD *key, char *s);
+void skipmask_add(char *mask);
 
 #if defined (HAVE_VSYSLOG) && defined (HAVE_FACILITYNAMES)
 
@@ -154,10 +179,11 @@ static void read_syslog_facility (KEYWORD *, char *);
 
 #endif
 
-#define DONT_CHECK 0x7fffffff
+#define DONT_CHECK 0x7fffffffl
 
 KEYWORD keywords[] =
 {
+  {"passwords", passwords, NULL, 0, 0},
   {"include", include, NULL, 0, 0},
   {"log", read_string, logpath, 'f', 0},
   {"loglevel", read_int, &loglevel, 0, DONT_CHECK},
@@ -182,7 +208,9 @@ KEYWORD keywords[] =
   {"maxclients", read_int, &max_clients, 0, DONT_CHECK},
   {"inbound", read_string, inbound, 'd', 0},
   {"inbound-nonsecure", read_string, inbound_nonsecure, 'd', 0},
+  {"temp-inbound", read_string, temp_inbound, 'd', 0},
   {"node", read_node_info, NULL, 0, 0},
+  {"defnode", read_node_info, NULL, 1, 0},
   {"kill-dup-partial-files", read_bool, &kill_dup_partial_files, 0, 0},
   {"kill-old-partial-files", read_int, &kill_old_partial_files, 1, DONT_CHECK},
   {"kill-old-bsy", read_int, &kill_old_bsy, 1, DONT_CHECK},
@@ -198,19 +226,31 @@ KEYWORD keywords[] =
   {"hold-skipped", read_int, &hold_skipped, 0, DONT_CHECK},
   {"backresolv", read_bool, &backresolv, 0, 0},
   {"pid-file", read_string, pid_file, 'f', 0},
+#ifdef HTTPS
+  {"proxy", read_string, proxy, 0, MAXHOSTNAMELEN + 40},
+  {"socks", read_string, socks, 0, MAXHOSTNAMELEN + 40},
+#endif
 #if defined (HAVE_VSYSLOG) && defined (HAVE_FACILITYNAMES)
   {"syslog", read_syslog_facility, &syslog_facility, 0, 0},
 #endif
   {"ftrans", read_rfrule, NULL, 0, 0},
   {"send-if-pwd", read_bool, &send_if_pwd, 0, 0},
   {"root-domain", read_string, root_domain, 0, MAXHOSTNAMELEN},
+  {"prescan", read_bool, &prescan, 0, 0},
+  {"connect-timeout", read_int, &connect_timeout, 0, DONT_CHECK},
+#ifdef MAILBOX
+  {"filebox", read_string, tfilebox, 'd', 0},
+  {"brakebox", read_string, bfilebox, 'd', 0},
+  {"deletebox", read_bool, &deleteablebox, 0, 0},
+#endif
+  {"skipmask", read_skipmask, NULL, 0, 0},
   {NULL, NULL, NULL, 0, 0}
 };
 
 #define TEST(var) if (!*var) Log (0, "%s: "#var" should be defined", path)
 
 void readcfg0 (char *_path);
-void debug_readcfg ();
+void debug_readcfg (void);
 
 /*
  * Parses and reads _path as config.file
@@ -246,6 +286,7 @@ void readcfg0 (char *_path)
   FILE *in;
   char buf[MAXCFGLINE + 1];
   char *w;
+  extern int checkcfg_flag;
 
   line = 0;
   path = _path;
@@ -253,6 +294,26 @@ void readcfg0 (char *_path)
   if ((in = fopen (path, "r")) == 0)
     Log (0, "%s: %s", path, strerror (errno));
 
+  if (checkcfg_flag)
+  {
+    struct conflist_type *pc;
+    
+    if (config_list)
+    {
+      for (pc = config_list; pc->next; pc = pc->next);
+      pc->next = xalloc(sizeof(*pc));
+      pc = pc->next;
+    }
+    else
+    {
+      config_list = xalloc(sizeof(*pc));
+      pc = config_list;
+    }
+    pc->next = NULL;
+    pc->path = xstrdup(path);
+    pc->mtime = 0;
+  }
+
   while (!feof (in))
   {
     if (!fgets (buf, sizeof (buf), in))
@@ -312,6 +373,44 @@ static void include (KEYWORD *key, char *s)
   }
 }
 
+static void passwords (KEYWORD *key, char *s)
+{
+  FILE *in;
+  char buf[MAXCFGLINE + 1];  
+  char *w = getword(s, 2);
+  FTN_ADDR fa;
+
+  if(!w) 
+    Log (0, "%s: %i: password filename expected", path, line);
+  if((in=fopen(w, "rt"))==NULL)
+    Log (0, "%s: %i: unable to open password file (%s)", path, line, w);
+  free(w);
+
+  while (!feof (in))
+  {
+    if (!fgets (buf, sizeof (buf), in))
+      break;
+    for(w=buf;isspace(w[0]);w++);  /* skip spaces */
+    if(w!=buf) strcpy(buf, w); 
+    for(w=buf;(w[0])&&(!isspace(w[0]));w++);
+    while(isspace(w[0]))           /* go to the password */
+    {
+      w[0]=0;
+      w++;
+    }
+    if((!w[0])||(!parse_ftnaddress (buf, &fa))) 
+      continue;     /* Do not process if any garbage found */
+    exp_ftnaddress (&fa);
+    strcpy(buf, w);
+    for(w=buf;(w[0])&&(!isspace(w[0]));w++);
+    w[0]=0;
+    if (!add_node (&fa, NULL, buf, '-', NULL, NULL,
+                   NR_USE_OLD, ND_USE_OLD, CRYPT_USE_OLD, 0, 0))
+      Log (0, "%s: add_node() failed", w[0]);
+  }
+  fclose(in);
+}
+
 static void read_aka_list (KEYWORD *key, char *s)
 {
   int i;
@@ -420,12 +519,19 @@ static void read_node_info (KEYWORD *key, char *s)
 {
 #define ARGNUM 6
   char *w[ARGNUM], *tmp;
-  int i, j, NR_flag = NR_USE_OLD;
+  int i, j, NR_flag = NR_USE_OLD, ND_flag = ND_USE_OLD;
+  int MD_flag = 0, crypt_flag = CRYPT_USE_OLD, restrictIP = 0;
   FTN_ADDR fa;
 
   memset (w, 0, sizeof (w));
   i = 0;			       /* index in w[] */
   j = 2;			       /* number of word in the source string */
+  
+  if(key->option1) /* defnode */
+  {
+	  w[i++]=xstrdup("0:0/0.0@defnode");
+	  havedefnode=1;
+  }
 
   while (1)
   {
@@ -436,8 +542,21 @@ static void read_node_info (KEYWORD *key, char *s)
     {
       if (tmp[1] != '\0')
       {
-	if (STRICMP (tmp, "-nr") == 0)
+        if (STRICMP (tmp, "-md") == 0)
+          MD_flag = 1;
+        else if (STRICMP (tmp, "-nomd") == 0)
+          MD_flag = (-1);
+        else if (STRICMP (tmp, "-nr") == 0)
 	  NR_flag = NR_ON;
+	else if (STRICMP (tmp, "-nd") == 0)
+	{
+	  NR_flag = NR_ON;
+	  ND_flag = ND_ON;
+	}
+	else if (STRICMP (tmp, "-ip") == 0)
+	  restrictIP = 1;
+	else if (STRICMP (tmp, "-crypt") == 0)
+	  crypt_flag = CRYPT_ON;
 	else
 	  Log (0, "%s: %i: %s: unknown option for `node' keyword", path, line, tmp);
       }
@@ -465,7 +584,8 @@ static void read_node_info (KEYWORD *key, char *s)
   check_dir_path (w[4]);
   check_dir_path (w[5]);
 
-  if (!add_node (&fa, w[1], w[2], w[3] ? w[3][0] : '-', w[4], w[5], NR_flag))
+  if (!add_node (&fa, w[1], w[2], (char)(w[3] ? w[3][0] : '-'), w[4], w[5],
+		 NR_flag, ND_flag, crypt_flag, MD_flag, restrictIP))
     Log (0, "%s: add_node() failed", w[0]);
 
   for (i = 0; i < ARGNUM; ++i)
@@ -475,6 +595,51 @@ static void read_node_info (KEYWORD *key, char *s)
 #undef ARGNUM
 }
 
+/*
+ *  Gets hostname/portnumber for ``n''-th host in ``src'' string (1 ... N)
+ *    <src> = <host> [ "," <src> ]
+ *    <host> = "*"
+ *    <host> = <hostname> [ ":" <service> ]
+ *
+ *  "*" will expand in corresponding domain name for ``fn''
+ *                        (2:5047/13 --> "f13.n5047.z2.fidonet.net.")
+ *
+ *  ``Host'' should contain at least MAXHOSTNAMELEN bytes.
+ *
+ *  Returns 0 on error, -1 on EOF, 1 otherwise
+ */
+int get_host_and_port (int n, char *host, unsigned short *port, char *src, FTN_ADDR *fa)
+{
+  int rc = 0;
+  char *s = getwordx2 (src, n, 0, ",;", "");
+
+  if (s)
+  {
+    char *t = strchr (s, ':');
+
+    if (t)
+      *t = 0;
+
+    if (!strcmp (s, "*"))
+      ftnaddress_to_domain (host, fa);
+    else
+      strnzcpy (host, s, MAXHOSTNAMELEN);
+
+    if (!t)
+    {
+      *port = oport;
+      rc = 1;
+    }
+    else if ((*port = find_port (t + 1)) != 0)
+      rc = 1;
+
+    free (s);
+  }
+  else
+    rc = -1;
+  return rc;
+}
+
 /*
  * Read a string (key->option1 == 0)
  * or a directory name (key->option1 == 'd')
@@ -492,7 +657,7 @@ static void read_string (KEYWORD *key, char *s)
   if (getword (s, 3) != NULL)
     Log (0, "%s: %i: extra arguments for `%s'", path, line, key->key);
 
-  strnzcpy (target, w, key->option1 == 0 ? key->option2 : MAXPATHLEN);
+  strnzcpy (target, w, key->option1 == 0 ? (size_t)key->option2 : MAXPATHLEN);
   free (w);
 
   if (key->option1 != 0)
@@ -517,7 +682,7 @@ static void read_string (KEYWORD *key, char *s)
 
 static void read_int (KEYWORD *key, char *s)
 {
-  int *target = (int *) (key->var);
+  long *target = (long *) (key->var);
   char *w;
 
   if ((w = getword (s, 2)) == NULL)
@@ -572,6 +737,17 @@ static void read_rfrule (KEYWORD *key, char *s)
     Log (0, "%s: %i: the syntax is incorrect", path, line);
 }
 
+static void read_skipmask (KEYWORD *key, char *s)
+{
+  char *w;
+  int i;
+
+  for (i=2; (w = getword (s, i)) != NULL; i++)
+    skipmask_add (w);
+  if (i == 2)
+    Log (0, "%s: %i: the syntax is incorrect", path, line);
+}
+
 static void read_bool (KEYWORD *key, char *s)
 {
   if (getword (s, 2) == 0)
@@ -607,6 +783,12 @@ static void read_flag_exec_info (KEYWORD *key, char *s)
     }
     if (key->option1 == 'e')
     {
+      if(*path == '!')
+      {
+        tmp->imm = 1;		       /* Immediate flag */
+        tmp->command = path + 1;
+      }
+      else
       tmp->command = path;
       tmp->path = 0;
     }
