@@ -12,9 +12,12 @@
  */
 
 /*
- * $Id: readflo.c,v 1.1 1997/03/28 06:20:30 mff Exp mff $
+ * $Id: readflo.c,v 2.0 2001/01/10 12:12:39 gul Exp $
  *
  * $Log: readflo.c,v $
+ * Revision 2.0  2001/01/10 12:12:39  gul
+ * Binkd is under CVS again
+ *
  * Revision 1.1  1997/03/28  06:20:30  mff
  * Initial revision
  *
@@ -89,7 +92,7 @@ char *trans_flo_line (char *s)
     strnzcpy (buf, s, MAXPATHLEN);
     for (curr = rf_rules; curr; curr = curr->next)
     {
-      w = ed (buf, curr->from, curr->to);
+      w = ed (buf, curr->from, curr->to, NULL);
       strnzcpy (buf, w, MAXPATHLEN);
       free (w);
     }
