@@ -12,9 +12,12 @@
  */
 
 /*
- * $Id: srif.h,v 1.3 1997/10/23 03:37:25 mff Exp mff $
+ * $Id: srif.h,v 2.0 2001/01/10 12:12:39 gul Exp $
  *
  * $Log: srif.h,v $
+ * Revision 2.0  2001/01/10 12:12:39  gul
+ * Binkd is under CVS again
+ *
  * Revision 1.3  1997/10/23  03:37:25  mff
  * *** empty log message ***
  *
@@ -25,6 +28,9 @@
 #define _srif_h
 
 #include "ftnq.h"
+#include "readcfg.h"
+#include "prothlp.h"
+#include "protoco2.h"
 
 typedef struct _EVT_FLAG EVT_FLAG;
 struct _EVT_FLAG
@@ -42,14 +48,14 @@ extern EVT_FLAG *evt_flags;
 /*
  * Tests if filename matches any of EVT_FLAG's patterns.
  */
-void evt_test (char *filename);
+int evt_test (char *filename);
 
 /*
  * Runs external programs using S.R.I.F. interface
  * if the name matches one of our "exec"'s
  */
 FTNQ *evt_run (FTNQ *q, char *filename0, FTN_ADDR *fa, 
-               int nfa, int prot, int listed, char *peer_name);
+               int nfa, int prot, int listed, char *peer_name, STATE *st);
 
 /*
  * Sets flags for all matched with evt_test events
