@@ -14,9 +14,12 @@
  */
 
 /*
- *  $Id: sys.h,v 1.3 1997/10/23 03:34:15 mff Exp mff $
+ * $Id: sys.h,v 2.0 2001/01/10 12:12:39 gul Exp $
+ *
+ * $Log: sys.h,v $
+ * Revision 2.0  2001/01/10 12:12:39  gul
+ * Binkd is under CVS again
  *
- *  $Log: sys.h,v $
  * Revision 1.3  1997/10/23  03:34:15  mff
  * many, many changes (forget to ci a version or two)
  *
@@ -34,16 +37,12 @@
   #include <io.h>
 #endif
 
-#if defined(NT) || defined(W95)
+#if defined(WIN32)
   #include <windows.h>
   #include <winsock.h>
 #endif
 
 #ifdef OS2
-  #ifdef __WATCOMC__
-    #define __IBMC__ 0
-    #define __IBMCPP__ 0
-  #endif
   #define INCL_DOS
   #define INCL_ERRORS
   #include <os2.h>
@@ -51,15 +50,18 @@
 #endif
 
 #ifdef HAVE_THREADS
-  #include <dos.h>
+  #ifndef __IBMC__
+    #include <dos.h>
+  #endif
   #include <process.h>
   #if defined(OS2)
     int gettid ();
     #define PID() gettid()
-  #elif defined(NT)
-    #define PID() ((int)GetCurrentThreadId())
-  #elif defined(W95)
+  #elif defined(WIN32)
     #define PID() ((int)(0xffff & GetCurrentThreadId()))
+  #elif defined(DOS)
+    int getpid ();
+    #define PID() getpid()
   #else
     #define PID() ((int)gettid())
   #endif
@@ -85,6 +87,18 @@
   #define _beginthread(a,b,c,d) _beginthread(a,c,d)
 #endif
 
+#ifdef DOS
+  void sleep(unsigned seconds);
+  #define _beginthread(a,b,c,d) a(d)
+  #define _endthread()
+  //_beginthread(void (* __thread)(void *),void * z,unsigned stk,void *arg);
+#endif
+
+#ifdef __IBMC__
+  #include <errno.h>
+  #define ENOTDIR EBADTYPE
+#endif
+
 typedef unsigned char u8;
 typedef unsigned short u16;
 typedef unsigned long u32;
@@ -97,7 +111,7 @@ unsigned long getfree (char *path);
 /*
  * Set up break handler, set up exit list if needed
  */
-int set_break_handlers ();
+int set_break_handlers (void);
 
 /*
  * Runs a new thread or forks
