@@ -12,9 +12,12 @@
  */
 
 /*
- * $Id: tools.c,v 1.18 1998/05/05 04:37:54 mff Exp mff $
+ * $Id: tools.c,v 2.0 2001/01/10 12:12:39 gul Exp $
  *
  * $Log: tools.c,v $
+ * Revision 2.0  2001/01/10 12:12:39  gul
+ * Binkd is under CVS again
+ *
  * Revision 1.18  1998/05/05  04:37:54  mff
  * Now we add trailing '\n' to pids in pidfiles, added 2nd arg to strquote(),
  * added strdequote(), strwipe() does not touch ' ' and 0xff any more,
@@ -90,6 +93,10 @@
 #include "readcfg.h"
 #include "sem.h"
 
+#ifdef WIN32
+extern int isService;
+#endif
+
 /*
  * Lowercase the string
  */
@@ -98,7 +105,7 @@ char *strlower (char *s)
   int i;
 
   for (i = 0; s[i]; ++i)
-    s[i] = tolower (s[i]);
+    s[i] = (char) tolower (s[i]);
   return s;
 }
 
@@ -159,16 +166,16 @@ int mkpath (char *s)
   return mkpath0 (path);
 }
 
-long rnd ()
+unsigned long rnd ()
 {
   static int i;
 
   if (!i)
   {
     i = 1;
-    srand (time (0));
+    srand ((unsigned int)time (0));
   }
-  return time (0) + rand ();
+  return (time (0) + rand ()) & 0xFFFFFFFFul;
 }
 
 /*
@@ -186,15 +193,22 @@ int create_empty_sem_file (char *name)
 
 int create_sem_file (char *name)
 {
-  FILE *f;
-  int h;
+  int h, i;
+  char buf[10];
 
   if ((h = open (name, O_RDWR | O_CREAT | O_EXCL, 0666)) == -1)
+  { Log (5, "Can't create %s: %s", name, strerror(errno));
     return 0;
-  if ((f = fdopen (h, "w")) == 0)
-    return 0;
-  fprintf (f, "%i\n", (int) getpid ());
-  fclose (f);
+  }
+  sprintf (buf, "%i\n", (int) getpid ());
+  if ((i = write(h, buf, strlen(buf))) != (int)strlen(buf))
+  { if (i == -1)
+      Log (2, "Can't write to %s (handle %d): %s", name, h, strerror(errno));
+    else
+      Log (2, "Can't write %d bytes to %s, wrote only %d", strlen(buf), name, i);
+  }
+  close (h);
+  Log (5, "created %s", name);
   return 1;
 }
 
@@ -221,11 +235,12 @@ void Log (int lev, char *s,...)
   struct tm *tm;
   va_list ap;
   static const char *marks = "!?+-";
-  char ch = (0 <= lev && lev < (int) strlen (marks)) ? marks[lev] : ' ';
+  char ch = (char)((0 <= lev && lev < (int) strlen (marks)) ? marks[lev] : ' ');
 
-  if (first_time)
+  if (first_time == 1)
   {
     InitSem (&LSem);
+    first_time = 2;
   }
 
   time (&t);
@@ -251,6 +266,7 @@ void Log (int lev, char *s,...)
     FILE *logfile = 0;
     int i;
 
+    LockSem (&LSem);
     for (i = 0; logfile == 0 && i < 10; ++i)
     {
       logfile = fopen (logpath, "a");
@@ -272,7 +288,18 @@ void Log (int lev, char *s,...)
     }
     else
       fprintf (stderr, "Cannot open %s: %s!\n", logpath, strerror (errno));
+    ReleaseSem (&LSem);
   }
+#ifdef WIN32
+  if((!lev)&&(isService))
+  {
+    char tmp[256];
+    va_start (ap, s);
+    vsprintf (tmp, s, ap);
+    va_end (ap);
+    MessageBox(NULL, tmp, MYNAME, MB_OK|MB_ICONSTOP|0x00200000L|MB_SYSTEMMODAL|MB_SETFOREGROUND);
+  }
+#endif
 
 #if defined (HAVE_VSYSLOG) && defined (HAVE_FACILITYNAMES)
   if (lev <= loglevel && syslog_facility >= 0)
@@ -311,7 +338,9 @@ void Log (int lev, char *s,...)
     exit (1);
 
 #if defined(EMX) || defined(__WATCOMC__)
+/*
   assert (_heapchk () == _HEAPOK || _heapchk () == _HEAPEMPTY);
+*/
 #endif
 }
 
@@ -364,7 +393,8 @@ char *strquote (char *s, int flags)
   for (i = 0; *s; ++s)
   {
     if (((flags & SQ_CNTRL) && iscntrl (*s)) ||
-	((flags & SQ_SPACE) && isspace (*s)))
+	((flags & SQ_SPACE) && isspace (*s)) ||
+	(*s == '\\'))
     {
       sprintf (r + i, "\\x%02x", *(unsigned char *) s);
       i += 4;
@@ -389,7 +419,7 @@ char *strdequote (char *s)
     if (s[0] == '\\' && s[1] == 'x' && isxdigit (s[2]) && isxdigit (s[3]))
     {
 #define XD(x) (isdigit(x) ? ((x)-'0') : (tolower(x)-'a'+10))
-      r[i++] = XD (s[2]) * 16 + XD (s[3]);
+      r[i++] = (char) (XD (s[2]) * 16 + XD (s[3]));
       s += 4;
 #undef XD
     }
@@ -408,7 +438,11 @@ char *strwipe (char *s)
   int i;
 
   for (i = 0; s[i]; ++i)
-    if (iscntrl (s[i]) || s[i] == '\\' || s[i] == '/' || s[i] == ':')
+    if ((iscntrl (s[i]) || s[i] == '\\' || s[i] == '/' || s[i] == ':')
+#ifdef WIN32 /* ungly hack */
+        && (s[i] & 0x80 == 0)
+#endif
+       )
       s[i] = '_';
   return s;
 }
@@ -467,57 +501,91 @@ int parse_args (int argc, char *argv[], char *src, char *ID)
  */
 int touch (char *file, time_t t)
 {
+#ifndef OS2
   struct utimbuf utb;
 
   utb.actime = utb.modtime = t;
   return utime (file, &utb);
+#else /* FastEcho deletes *.bsy by ctime :-( */
+  APIRET r;
+  FILESTATUS3 buf;
+  struct stat st;
+
+  if ((r = stat(file, &st)) == 0)
+  {
+    struct tm *tm;
+#ifdef __WATCOMC__
+    struct tm stm;
+    tm = &stm;
+    _localtime(&t, tm);
+#else
+    tm=localtime(&t);
+#endif
+    buf.fdateCreation.day=buf.fdateLastAccess.day=buf.fdateLastWrite.day=
+        tm->tm_mday;
+    buf.fdateCreation.month=buf.fdateLastAccess.month=buf.fdateLastWrite.month=
+        tm->tm_mon+1;
+    buf.fdateCreation.year=buf.fdateLastAccess.year=buf.fdateLastWrite.year=
+        tm->tm_year-80;
+    buf.ftimeCreation.twosecs=buf.ftimeLastAccess.twosecs=buf.ftimeLastWrite.twosecs=
+        tm->tm_sec/2;
+    buf.ftimeCreation.minutes=buf.ftimeLastAccess.minutes=buf.ftimeLastWrite.minutes=
+        tm->tm_min;
+    buf.ftimeCreation.hours=buf.ftimeLastAccess.hours=buf.ftimeLastWrite.hours=
+        tm->tm_hour;
+    buf.cbFile = buf.cbFileAlloc = st.st_size;
+    buf.attrFile = FILE_ARCHIVED | FILE_NORMAL;
+    r=DosSetPathInfo(file, FIL_STANDARD, &buf, sizeof(buf), 0);
+    if (r == 32)
+      r = 0; /* Can't touch opened *.bsy */
+    if (r)
+      Log (1, "touch: DosSetPathInfo(%s) retcode %d", file, r);
+  }
+  return (r!=0);
+#endif
 }
 
 /*
  * Replaces all entries of a in s for b, returns edited line.
  * Returned value must be free()'d. Ignores case.
+ * size parameter can be used only if "s" was created by malloc()
  */
-char *ed (char *src, char *a, char *b)
+char *ed (char *s, char *a, char *b, unsigned int *size)
 {
-  int sz = strlen (src) + 1;
-  int pos;
-  int len_a = strlen (a);
-  int len_b = strlen (b);
-  char *r = xstrdup (src);
-  char *s1, *s2;
-
-  if (len_a != 0)
-    for (pos = 0;; pos += len_b)
-    {
-      s1 = strchr (r + pos, tolower (*a));
-      s2 = strchr (r + pos, toupper (*a));
-      if ((s1 = s1 && s2 ? min (s1, s2) : (s1 ? s1 : s2)) != 0)
-      {
-	if (!MEMICMP (s1, a, len_a))
-	{
-	  int new_sz = sz - len_a + len_b;
-
-	  pos = s1 - r;
-	  if (new_sz > sz)
-	  {
-	    r = xrealloc (r, new_sz);
-	    sz = new_sz;
-	  }
-	  memmove (r + pos + len_b, r + pos + len_a, strlen (r + pos + len_a) + 1);
-	  memcpy (r + pos, b, len_b);
-	  if (new_sz != sz)
-	  {
-	    r = xrealloc (r, new_sz);
-	    sz = new_sz;
-	  }
-	}
-	else
-	  ++pos;
-      }
-      else
-	break;
-    }
-  return r;
+   int i, j;
+   unsigned int sz;
+   int len_a=a?strlen(a):0;
+   int len_b=b?strlen(b):0;
+   char *r=s;
+   
+   if((!len_a)||(!s)) return r;
+   if(!size) 
+   {
+     sz=strlen(s)+1;
+     r=xstrdup(s);
+   }
+   else sz=*size;
+   for(i=j=0;i<(int)strlen(r);i++)
+   {
+     if(tolower(r[i])!=tolower(a[j++]))
+     {
+       j=0;
+       continue;
+     }
+     if(a[j]) continue;
+     if(strlen(r)-len_a+len_b>sz)
+     {
+       if(len_b<64) sz+=64;
+       else sz+=len_b;
+       r=xrealloc(r, sz);
+     }
+     i-=len_a-1;
+     memmove(r+i+len_b, r+i+len_a, strlen(r+i+len_a)+1);
+     if(len_b)
+       memcpy(r+i, b, len_b);
+   }
+   if(size) *size=sz;
+   return r;
 }
 
 /*
@@ -527,7 +595,7 @@ int delete (char *path)
 {
   int rc;
 
-  if ((rc = unlink (path)) == -1)
+  if ((rc = unlink (path)) != 0)
     Log (1, "error unlinking `%s': %s", path, strerror (errno));
   else
     Log (5, "unlinked `%s'", path);
