@@ -12,9 +12,12 @@
  */
 
 /*
- * $Id: tools.h,v 1.10 1997/10/23 03:26:47 mff Exp mff $
+ * $Id: tools.h,v 2.0 2001/01/10 12:12:39 gul Exp $
  *
  * $Log: tools.h,v $
+ * Revision 2.0  2001/01/10 12:12:39  gul
+ * Binkd is under CVS again
+ *
  * Revision 1.10  1997/10/23  03:26:47  mff
  * getword*() is in getw.h now
  *
@@ -31,7 +34,11 @@
 #define _tools_h
 
 #include <stdarg.h>
+#include <stddef.h>  /* for size_t */
 #include <sys/types.h>
+#ifdef VISUALCPP
+#include <malloc.h>  /* for size_t */
+#endif
 
 #include "getw.h"
 
@@ -50,8 +57,8 @@ void Log (int lev, char *s,...);
 /*
  * (xalloc.c) [Re]allocate memory or log error
  */
-void *xalloc (size_t size);
-void *xrealloc (void *ptr, size_t size);
+void *xalloc (unsigned long size);
+void *xrealloc (void *ptr, unsigned long size);
 void *xstrdup (const char *str);
 
 /*
@@ -102,7 +109,7 @@ int create_empty_sem_file (char *s);
 
 /*
  */
-long rnd ();
+unsigned long rnd (void);
 
 /*
  * Makes all dirs in the path
@@ -130,7 +137,7 @@ int touch (char *file, time_t t);
  * Replaces all entries of a in s for b, returns edited line.
  * Returned value must be free()'d. Ignores case.
  */
-char *ed (char *src, char *a, char *b);
+char *ed (char *src, char *a, char *b, unsigned int *size);
 
 /*
  * Remove/trucate a file, log this
@@ -141,7 +148,7 @@ int trunc (char *);
 /*
  * Get the string with OS name/version
  */
-char *get_os_string ();
+char *get_os_string (void);
 
 /*
  * Test netnames against some wildcards
